import pytest

from barbell2.lib import BasicShell


NR_RESULTS = 4


@pytest.fixture
def shell():
    shell = BasicShell()
    for i in range(NR_RESULTS):
        shell.add_result([i] * 5, desc='Array of numbers ({})'.format(i))
    return shell


def test_has_two_results(shell):
    assert len(shell.results.keys()) == NR_RESULTS


def test_results_have_correct_names(shell):
    for i in range(NR_RESULTS):
        assert 'result_{}'.format(i) in shell.results.keys()


def test_last_result_is_current(shell):
    last_key = sorted(shell.results.keys(), reverse=False)[-1]
    assert last_key == shell.current_key


def test_undo_redo(shell):
    key_list = sorted(shell.results.keys(), reverse=False)
    shell.do_undo()
    assert shell.current_key == key_list[-2]
    shell.do_redo()
    assert shell.current_key == key_list[-1]



import pydicom

from barbell2.utils import apply_window


class Dicom2Numpy:

    def __init__(self, dcm_file_path_or_obj):
        self.dcm_file_path_or_obj = dcm_file_path_or_obj
        self.npy_array = None
        self.window = None
        self.normalize_enabled = True

    def set_window(self, window):
        self.window = window

    def set_normalize_enabled(self, normalize_enabled):
        self.normalize_enabled = normalize_enabled

    def is_normalize_enabled(self):
        return self.normalize_enabled

    def execute(self):
        self.npy_array = None
        if isinstance(self.dcm_file_path_or_obj, str):
            p = pydicom.dcmread(self.dcm_file_path_or_obj)
        else:
            p = self.dcm_file_path_or_obj
        pixels = p.pixel_array
        print(f'dcm2npy: pixels shape = {pixels.shape}')
        self.npy_array = pixels.reshape(p.Rows, p.Columns)
        if self.is_normalize_enabled():
            b = p.RescaleIntercept
            m = p.RescaleSlope
            self.npy_array = m * self.npy_array + b
        if self.window is not None:
            self.npy_array = apply_window(self.npy_array, self.window)
        return self.npy_array


# x = '/Volumes/SEAGATE_RALPH/data/hpb/metabolicimaging/bodycomposition/data/neuro_mumc/Sandra/DCM_TAG_LIJST/19001.dcm'
# p = pydicom.dcmread(x, force=True)
# p.file_meta.TransferSyntaxUID = '1.2.840.10008.1.2'
# n = Dicom2Numpy(p)
# print(n.execute().shape)



import os
import pytest

from barbell2.dicomexplorer.dicomexplorer import DicomExplorer


COHORT_DIR = '/Volumes/USB_SECURE1/data/radiomics/projects/deepseg/data/mega/processed/NEWEPOC'
COHORT_DCM_FILE = '/Volumes/USB_SECURE1/data/radiomics/projects/deepseg/data/mega/processed/NEWEPOC/003001_pre_PV_L3.dcm'
COHORT_TAG_FILE = '/Volumes/USB_SECURE1/data/radiomics/projects/deepseg/data/mega/processed/NEWEPOC/003001_pre_PV_L3.tag'
COHORT_DCM_HEADER_NR_ENTRIES = 86
COHORT_NR_FILES = 156
DICOM_DICT_NR_ENTRIES = 4253
PATIENT_ID_NR_ENTRIES = 6


@pytest.fixture
def explorer():
    return DicomExplorer()


def test_load_dir(explorer):
    explorer.load_dir(COHORT_DIR, verbose=False)
    assert len(explorer.files) == COHORT_NR_FILES


def test_load_dicom_file(explorer):
    explorer.load_file(COHORT_DCM_FILE, verbose=False)
    assert len(explorer.files) == 1


def test_load_non_dicom_file(explorer):
    explorer.load_file(COHORT_TAG_FILE, verbose=False)
    assert len(explorer.files) == 0


def test_get_all_tags(explorer):
    outputs = explorer.get_tags(verbose=False)
    assert len(outputs) == DICOM_DICT_NR_ENTRIES


def test_get_patient_id_tag(explorer):
    outputs = explorer.get_tags(key_word='PatientID', verbose=False)
    assert len(outputs) == PATIENT_ID_NR_ENTRIES


def test_get_header(explorer):
    d = explorer.get_header(COHORT_DCM_FILE, verbose=False)
    assert len(list(d.elements())) == COHORT_DCM_HEADER_NR_ENTRIES


def test_get_pixels(explorer):
    p = explorer.get_pixel_data(COHORT_DCM_FILE)
    assert p.shape == (512, 512)



import pandas as pd
import data


class CastorExportFileLoader:

    def __init__(self, file_path):
        self.file_path = file_path
        self.data = None

    def load(self):
        print('Loading Castor data...')
        df = pd.read_excel(self.file_path, dtype=str, engine='openpyxl', sheet_name='Study results')
        print('Loading Castor data dictionary...')
        df_dict = pd.read_excel(self.file_path, dtype=str, engine='openpyxl', sheet_name='Study variable list')
        self.data = data.CastorData(df, df_dict)
        return self.data


class DicaExportFileLoader:

    def __init__(self, file_path_data, file_path_dict):
        self.file_path_data = file_path_data
        self.file_path_dict = file_path_dict
        self.data = None

    def load(self):
        print('Loading DICA data...')
        df_pat = pd.read_excel(self.file_path_data, dtype=str, sheet_name='patient', engine='openpyxl', index_col='uri')
        df_trt = pd.read_excel(self.file_path_data, dtype=str, sheet_name='verrichting', engine='openpyxl', index_col='uri')
        df_com = pd.read_excel(self.file_path_data, dtype=str, sheet_name='comorbiditeiten', engine='openpyxl', index_col='uri')
        print('Loading DICA data dictionary...')
        df_dict_pat = pd.read_excel(self.file_path_dict, dtype=str, sheet_name='patient', engine='openpyxl')
        df_dict_trt = pd.read_excel(self.file_path_dict, dtype=str, sheet_name='verrichting', engine='openpyxl')
        df_dict_com = pd.read_excel(self.file_path_dict, dtype=str, sheet_name='comorbiditeiten', engine='openpyxl')
        self.data = data.DicaData(df_pat, df_trt, df_com, df_dict_pat, df_dict_trt, df_dict_com)
        return self.data
